const slugify = require("slugify");

const path = require("path");
const fs = require("fs");

const ServiceModel = require("../models/ServiceModel");
const UsereModel = require("../models/UserModel");

const OfferModel = require("../models/OfferModel");
const ApiError = require("../utils/apiError");

const { uploadSingleImage } = require("../Middlewares/uploadImageMiddleware");
const { uploadMixOfImages } = require("../Middlewares/uploadImageMiddleware");
const ServicesResource = require("../utils/resources/ServicesResource");
const MyServicesResource = require("../utils/resources/auth/ServicesResource");
const { send_notification } = require("./NotificationController");

exports.UploadImage = uploadSingleImage("image", "images/services");
exports.MultiImages = uploadMixOfImages("images", "images/services");

exports.GetServicesOfUser = async (req, res) => {
  const services = await ServiceModel.find({ user: req.user._id }).sort({
    createdAt: -1,
  });
  if (!services) {
    const error = new Error("Service not founded ");
    error.statusCode = 422;
    throw error;
  } else {
    const servicesWithOffers = await Promise.all(
      services.map(async (service) => {
        const offers = await OfferModel.find({ service: service._id }).populate(
          "user"
        );
        return { ...service.toObject(), offers }; // Merge service and offers
      })
    );
    const data = await MyServicesResource.collection(servicesWithOffers);
    res.status(200).json({ data: data });
  }
};

exports.GetServiceById = async (req, res, next) => {
  const { id } = req.params;
  const Service = await ServiceModel.findById(id).populate("user");
  if (!Service) {
    next(new ApiError(`not found ${req.originalUrl}`, 400));
  }
  const data = await new ServicesResource(Service).toJSON();
  res.status(200).json({ data });
};

exports.AddService = async (req, res) => {
  const { description, price, category_id, locations } = req.body;
  let images = [];
  if (req.files) {
    images = req.files ? req.files.map((file) => file.path) : [];
  }

  let parsedLocations = [];
  if (locations) {
    try {
      parsedLocations = JSON.parse(locations);
    } catch (error) {
      return res
        .status(400)
        .json({ success: false, message: " Wrong Location Format " });
    }
  }

  try {
    const add = await ServiceModel.create({
      description,
      price,
      images,
      category_id,
      locations: parsedLocations,
      user: req.user._id,
    });
    const Service = await ServiceModel.findById(add._id).populate("user");
    const data = await new ServicesResource(Service).toJSON();
    const io = req.app.get("socketio");
    const serviceNamespace = io.of("/SocketService");
    const categoryRoom = `category_${category_id}`;
    serviceNamespace.to(categoryRoom).emit("NewService", { data });
    await send_notification(req.user._id, null, null, add._id, "new_service");
    res.status(200).json({ data: add });
  } catch (error) {
    if (error.code === 11000) {
      // MongoDB duplicate key error code is 11000
      res.status(409).json({
        message: "Service with the same name already exists",
        error: error.message,
      });
    }
  }
};

exports.NewService = async (req, res) => {
  const selectedCategories = req.user.selectedCategories;
  const userId = req.user._id; //

  const offeredServiceIds = await OfferModel.find({
    user: userId,
    OfferStatus: { $nin: ["canceled", "rejected"] },
  }).distinct("service");

  const services = await ServiceModel.where("category_id")
    .in(selectedCategories)
    .where("user")
    .ne(userId)
    .where("_id")
    .nin(offeredServiceIds)
    .where("ServiceStatus")
    .equals("active")
    .sort({
      createdAt: -1,
    })
    .populate("user");
  const data = await ServicesResource.collection(services);
  res.status(200).json({
    success: true,
    services: data,
  });
};

exports.DeleteService = async (socket, serviceId) => {
  const Service = await ServiceModel.findOneAndUpdate(
    { _id: serviceId },
    { ServiceStatus: "canceled" },
    { new: true, runValidators: true }
  );

  if (!Service) {
    const error = new Error("Service not founded ");
    error.statusCode = 422;
    throw error;
  }

  const ServiceOwner = await this.GetServicesOfUserocket(Service.user);
  socket.to(`user_${Service.user}`).emit("MyServices", ServiceOwner);

  socket.to(`category_${Service.category_id}`).emit("DeleteService", serviceId);


};

exports.NewServiceSocket = async (selectedCategories, userId) => {
  const offeredServiceIds = await OfferModel.find({
    user: userId,
    OfferStatus: { $nin: ["canceled"] },
  }).distinct("service");

  const services = await ServiceModel.where("category_id")
    .in(selectedCategories)
    .where("user")
    .ne(userId)
    .where("_id")
    .nin(offeredServiceIds)
    .where("ServiceStatus")
    .equals("active")
    .sort({
      createdAt: -1,
    })
    .populate("user");
  const data = await ServicesResource.collection(services);
  return data;
};

exports.GetServicesOfUserocket = async (userId, status) => {
  const services = await ServiceModel.find({
    user: userId,
    ...(status && {
      ServiceStatus: { $in: Array.isArray(status) ? status : [status] },
    }),
  }).sort({
    createdAt: -1,
  });
  if (!services.length) return [];

  const data = await MyServicesResource.collection(services);

  const servicesWithPendingCounts = await Promise.all(
    data.map(async (service) => {
      const OffersCount = await OfferModel.countDocuments({
        service: service.id,
      });

      return {
        ...service,
        OffersCount,
      };
    })
  );

  return servicesWithPendingCounts;
};

exports.GetDetails = async (serviceId) => {
  const service = await ServiceModel.findById(serviceId);
  if (!service) {
    throw new Error("Service not found");
  }

  const offers = await OfferModel.find({ service: service._id }).populate(
    "user"
  );
  const serviceWithOffers = {
    ...service.toObject(),
    offers,
  };

  const data = await new MyServicesResource(serviceWithOffers).toJSON();

  return data;
};
