const mongoose = require("mongoose");
const slugify = require("slugify");

const CategorySchema = new mongoose.Schema(
  {
    name_en: {
      type: String,
      required: [true, "Name is required in English"],
      minlength: [3, "Too short name"],
    },
    name_ar: {
      type: String,
      required: [true, "Name is required in Arabic"],
      minlength: [3, "Too short name"],
    },
    slug: {
      type: String,
      unique: true,
      lowercase: true,
    },
    image: {
      type: String,
    },
    status: {
      type: Boolean,
      default: true,
    },
    numberOfLocations: {
      type: Number,
      enum: [0, 1, 2],
      default: 1,
    },
  },
  { timestamps: true }
);

// Middleware to generate a unique slug
CategorySchema.pre("save", async function (next) {
  if (!this.name_en) {
    this.name_en = this.name_ar;
  }

  let newSlug = slugify(this.name_en, { lower: true });

  // Check if the slug already exists
  const existingCategory = await mongoose.model("Category").findOne({ slug: newSlug });

  if (existingCategory) {
    newSlug = `${newSlug}-${Date.now()}`;
  }

  this.slug = newSlug;
  next();
});

// Static method to check for duplicates before creating a category
CategorySchema.statics.createCategory = async function (categoryData) {
  if (!categoryData.name_en || !categoryData.name_ar) {
    throw new Error("Both name_en and name_ar are required.");
  }
  const existing = await this.findOne({
    $or: [{ name_en: categoryData.name_en }, { name_ar: categoryData.name_ar }],
  });

  if (existing) {
    throw new Error("Category with this name already exists.");
  }

  return this.create(categoryData);
};

const CategoryModel = mongoose.model("Category", CategorySchema);
module.exports = CategoryModel;
