const { check } = require("express-validator");
const validatorMiddleware = require("../../Middlewares/validatorMiddleware");
const ServiceModel = require("../../models/ServiceModel");
const OfferModel = require("../../models/OfferModel");

exports.AddOfferValidate = [
  check("serviceId")
    .isMongoId()
    .withMessage("Invalid service ID")
    .custom(async (serviceId, { req }) => {
      const userId = req.user._id;

      // Check if service exists and is active
      const service = await ServiceModel.findOne({
        _id: serviceId,
        ServiceStatus: "active",
      });
      if (!service) {
        throw new Error("Service not found or not active");
      }

      // Prevent users from making offers on their own services
      if (service.user.toString() === userId.toString()) {
        throw new Error("You cannot create an offer for your own service.");
      }

      // Check if an offer already exists for this service by the user
      const existingOffer = await OfferModel.findOne({
        user: userId.toString(),
        service: serviceId,
        OfferStatus: {
          $in: ["pending", "rejected", "accepted", "ended", "not_accepted"],
        },
      });
      console.log("existingOffer", existingOffer);
      console.log("userId", userId);
      console.log("serviceId", serviceId);
      if (existingOffer) {
        throw new Error(
          `You already have an offer for this service. Please remove it and try again.${existingOffer}`
        );
      }

      return true;
    }),

  check("price")
    .notEmpty()
    .withMessage("Price is required")
    .isNumeric()
    .withMessage("Make sure to send a number in the price field"),

  validatorMiddleware,
];

exports.GetOfferValidate = [
  check("id")
    .isMongoId()
    .withMessage("Invalid service ID")
    .custom(async (id, { req }) => {
      const userId = req.user._id;

      // Check if service exists
      const service = await ServiceModel.findOne({ _id: id });

      if (!service) {
        throw new Error("Service not found.");
      }

      // Check if the user is the owner of the service
      if (service.user.toString() !== userId.toString()) {
        throw new Error(
          "You cannot view offers for a service that is not yours."
        );
      }

      return true;
    }),

  validatorMiddleware,
];

exports.StatusOfferValidate = [
  check("id")
    .isMongoId()
    .withMessage("Invalid offer ID")
    .custom(async (id, { req }) => {
      const userId = req.user._id;
      console.log("userId:", userId);
      const offer = await OfferModel.findOne({ _id: id })
        .populate({
          path: "service",
          populate: {
            path: "user",
          },
        })
        .populate("user");
      if (!offer) {
        throw new Error("Offer not found.");
      }
      req.offer = offer;
      const { status } = req.body;
      if (status === "canceled") {
        console.log("offer.user._id.toString():", offer.user._id.toString());
        if (offer.user._id.toString() !== userId.toString()) {
          throw new Error(
            "You cannot cancel offers for a service that is not yours."
          );
        }

        if (offer.OfferStatus !== "pending") {
          throw new Error(
            `You cannot cancele offers that are ${offer.OfferStatus}.`
          );
        }
      } else if (["rejected", "accepted", "ended"].includes(status)) {
        const service = await ServiceModel.findById(offer.service);
        if (!service) {
          throw new Error("Service not found.");
        }
        if (service.user.toString() !== userId.toString()) {
          throw new Error(
            "You cannot update the status of offers for a service that is not yours."
          );
        }
      }
      return true;
    }),
  check("status")
    .isIn(["canceled", "rejected", "accepted"])
    .withMessage("Invalid status value."),
  validatorMiddleware,
];
